@@ -5,6 +5,7 @@ use std::sync::RwLock;
 use std::sync::{Arc, Mutex};
 use std::time::Duration;
 
+use crate::RequestId;
 use crate::adapters::{EntriesOnly, IntoAdapterVec};
 use crate::controls_impl::IntoRawControlVec;
 use crate::exop::Exop;
@@ -14,17 +15,17 @@ use crate::result::{
     CompareResult, ExopResult, LdapError, LdapResult, LdapResultExt, Result, SearchResult,
 };
 use crate::search::{Scope, SearchOptions, SearchStream};
-use crate::RequestId;
 
 use lber::common::TagClass;
 use lber::structures::{Boolean, Enumerated, Integer, Null, OctetString, Sequence, Set, Tag};
 
 #[cfg(feature = "gssapi")]
-use cross_krb5::{ClientCtx, InitiateFlags, K5Ctx, Step};
+use cross_krb5::{ClientCtx, Cred, InitiateFlags, K5Ctx, Step};
 use tokio::sync::{mpsc, oneshot};
 use tokio::time;
 
 /// SASL bind exchange wrapper.
+#[allow(dead_code)]
 #[derive(Clone, Debug)]
 pub(crate) struct SaslCreds(pub Option<Vec<u8>>);
 
@@ -82,7 +83,7 @@ pub struct Ldap {
     pub(crate) sasl_param: Arc<RwLock<(bool, u32)>>, // sasl_wrap, sasl_max_send
     #[cfg(feature = "gssapi")]
     pub(crate) client_ctx: Arc<Mutex<Option<ClientCtx>>>,
-    #[cfg(feature = "gssapi")]
+    #[cfg(any(feature = "gssapi", feature = "ntlm"))]
     pub(crate) tls_endpoint_token: Arc<Option<Vec<u8>>>,
     pub(crate) has_tls: bool,
     pub timeout: Option<Duration>,
@@ -101,7 +102,7 @@ impl Clone for Ldap {
             sasl_param: self.sasl_param.clone(),
             #[cfg(feature = "gssapi")]
             client_ctx: self.client_ctx.clone(),
-            #[cfg(feature = "gssapi")]
+            #[cfg(any(feature = "gssapi", feature = "ntlm"))]
             tls_endpoint_token: self.tls_endpoint_token.clone(),
             has_tls: self.has_tls,
             last_id: 0,
@@ -144,6 +145,12 @@ fn sasl_bind_req(mech: &str, creds: Option<&[u8]>) -> Tag {
     })
 }
 
+#[cfg(feature = "gssapi")]
+enum GssapiCred {
+    Default,
+    Supplied(Cred),
+}
+
 impl Ldap {
     fn next_msgid(&mut self) -> i32 {
         let mut msgmap = self.msgmap.lock().expect("msgmap mutex (inc id)");
@@ -283,29 +290,77 @@ impl Ldap {
     /// querying the ticket cache or the Kerberos servers. Therefore, the method should not
     /// be used in heavily concurrent contexts with frequent Bind operations.
     pub async fn sasl_gssapi_bind(&mut self, server_fqdn: &str) -> Result<LdapResult> {
+        self.gssapi_bind(server_fqdn, GssapiCred::Default).await
+    }
+
+    #[cfg_attr(docsrs, doc(cfg(feature = "gssapi")))]
+    #[cfg(feature = "gssapi")]
+    /// Do an SASL GSSAPI bind on the connection, using the supplied GSSAPI credentials and
+    /// `server_fqdn` for the LDAP server SPN. Aside from using the supplied credentials, this
+    /// method behaves identically as [`gssapi_bind()`](#method.gssapi_bind) (q.v.)
+    pub async fn sasl_gssapi_cred_bind(
+        &mut self,
+        cred: Cred,
+        server_fqdn: &str,
+    ) -> Result<LdapResult> {
+        self.gssapi_bind(server_fqdn, GssapiCred::Supplied(cred))
+            .await
+    }
+
+    #[cfg(feature = "gssapi")]
+    async fn gssapi_bind(&mut self, server_fqdn: &str, cred: GssapiCred) -> Result<LdapResult> {
         const LDAP_RESULT_SASL_BIND_IN_PROGRESS: u32 = 14;
         const GSSAUTH_P_NONE: u8 = 1;
         const GSSAUTH_P_PRIVACY: u8 = 4;
 
+        use either::Either;
+
         let mut spn = String::from("ldap/");
         spn.push_str(server_fqdn);
         let cti = if self.has_tls {
             let cbt = {
                 let mut cbt = Vec::from(&b"tls-server-end-point:"[..]);
-                if let Some(ref token) = self.tls_endpoint_token.as_ref() {
+                if let Some(token) = self.tls_endpoint_token.as_ref() {
                     cbt.extend(token);
                     Some(cbt)
                 } else {
                     None
                 }
             };
-            ClientCtx::new(InitiateFlags::empty(), None, &spn, cbt.as_deref())
+            match cred {
+                GssapiCred::Default => Either::Left(ClientCtx::new(
+                    InitiateFlags::empty(),
+                    None,
+                    &spn,
+                    cbt.as_deref(),
+                )),
+                GssapiCred::Supplied(cred) => {
+                    Either::Right(ClientCtx::new_with_cred(cred, &spn, cbt.as_deref()))
+                }
+            }
         } else {
-            ClientCtx::new(InitiateFlags::empty(), None, &spn, None)
+            match cred {
+                GssapiCred::Default => {
+                    Either::Left(ClientCtx::new(InitiateFlags::empty(), None, &spn, None))
+                }
+                GssapiCred::Supplied(cred) => {
+                    Either::Right(ClientCtx::new_with_cred(cred, &spn, None))
+                }
+            }
         };
-        let (client_ctx, token) =
-            cti.map_err(|e| LdapError::GssapiOperationError(format!("{:#}", e)))?;
-        let req = sasl_bind_req("GSSAPI", Some(&token));
+        let (client_ctx, token) = match cti {
+            Either::Left(cti) => cti
+                .map(|(c, t)| (c, Either::Left(t)))
+                .map_err(|e| LdapError::GssapiOperationError(format!("{:#}", e)))?,
+            Either::Right(cti) => cti
+                .map(|(c, t)| (c, Either::Right(t)))
+                .map_err(|e| LdapError::GssapiOperationError(format!("{:#}", e)))?,
+        };
+        let token = match token {
+            Either::Left(ref t) => t.as_ref(),
+            Either::Right(ref t) => t.as_ref(),
+        };
+        let req = sasl_bind_req("GSSAPI", Some(token));
         let ans = self.op_call(LdapOp::Single, req).await?;
         if (ans.0).rc != LDAP_RESULT_SASL_BIND_IN_PROGRESS {
             return Ok(ans.0);
@@ -322,7 +377,7 @@ impl Ldap {
             _ => {
                 return Err(LdapError::GssapiOperationError(String::from(
                     "GSSAPI exchange not finished or has an additional token",
-                )))
+                )));
             }
         };
         let req = sasl_bind_req("GSSAPI", None);
@@ -375,6 +430,84 @@ impl Ldap {
         Ok(res)
     }
 
+    #[cfg_attr(docsrs, doc(cfg(feature = "ntlm")))]
+    #[cfg(feature = "ntlm")]
+    /// Do an SASL GSS-SPNEGO bind with an NTLMSSP exchange on the connection. Username
+    /// and password must be provided, since the method is incapable of retrieving the
+    /// credentials associated with the login session (which would only work on Windows
+    /// anyway.) To specify the domain, incorporate it into the username, using the
+    /// `DOMAIN\user` or `user@DOMAIN` format.
+    ///
+    /// __Caveat:__ the connection cannot encrypted by NTLM "sealing". For encryption, use
+    /// TLS. A channel binding token is automatically sent on a TLS connection, if possible.
+    pub async fn sasl_ntlm_bind(&mut self, username: &str, password: &str) -> Result<LdapResult> {
+        const LDAP_RESULT_SASL_BIND_IN_PROGRESS: u32 = 14;
+
+        use sspi::{
+            AuthIdentity, AuthIdentityBuffers, BufferType, ClientRequestFlags, CredentialUse,
+            DataRepresentation, Ntlm, SecurityBuffer, SecurityStatus, Sspi, SspiImpl, Username,
+            builders::AcquireCredentialsHandleResult,
+        };
+
+        fn step(
+            ntlm: &mut Ntlm,
+            acq_creds: &mut AcquireCredentialsHandleResult<Option<AuthIdentityBuffers>>,
+            input: &[u8],
+        ) -> Result<Vec<u8>> {
+            let mut input = vec![SecurityBuffer::new(input.to_vec(), BufferType::Token)];
+            let mut output = vec![SecurityBuffer::new(Vec::new(), BufferType::Token)];
+            let mut builder = ntlm
+                .initialize_security_context()
+                .with_credentials_handle(&mut acq_creds.credentials_handle)
+                .with_context_requirements(ClientRequestFlags::ALLOCATE_MEMORY)
+                .with_target_data_representation(DataRepresentation::Native)
+                .with_input(&mut input)
+                .with_output(&mut output);
+            let result = ntlm
+                .initialize_security_context_impl(&mut builder)?
+                .resolve_to_result()?;
+            match result.status {
+                SecurityStatus::CompleteNeeded | SecurityStatus::CompleteAndContinue => {
+                    ntlm.complete_auth_token(&mut output)?
+                }
+                s => s,
+            };
+            Ok(output.swap_remove(0).buffer)
+        }
+
+        let mut ntlm = Ntlm::new();
+        let identity = AuthIdentity {
+            username: Username::parse(username).unwrap(),
+            password: password.to_string().into(),
+        };
+        let mut acq_creds = ntlm
+            .acquire_credentials_handle()
+            .with_credential_use(CredentialUse::Outbound)
+            .with_auth_data(&identity)
+            .execute(&mut ntlm)?;
+        let req = sasl_bind_req("GSS-SPNEGO", Some(&step(&mut ntlm, &mut acq_creds, &[])?));
+        let (res, _, token) = self.op_call(LdapOp::Single, req).await?;
+        if res.rc != LDAP_RESULT_SASL_BIND_IN_PROGRESS {
+            return Ok(res);
+        }
+        let token = match token.0 {
+            Some(token) => token,
+            _ => return Err(LdapError::NoNtlmChallengeToken),
+        };
+        if self.has_tls {
+            let mut cbt = Vec::from(&b"tls-server-end-point:"[..]);
+            if let Some(token) = self.tls_endpoint_token.as_ref() {
+                cbt.extend(token);
+                ntlm.set_channel_bindings(&cbt);
+            }
+        }
+        let req = sasl_bind_req(
+            "GSS-SPNEGO",
+            Some(&step(&mut ntlm, &mut acq_creds, &token)?),
+        );
+        Ok(self.op_call(LdapOp::Single, req).await?.0)
+    }
+
     /// Perform a Search with the given base DN (`base`), scope, filter, and
     /// the list of attributes to be returned (`attrs`). If `attrs` is empty,
     /// or if it contains a special name `*` (asterisk), return all (user) attributes.
@@ -694,7 +827,7 @@ impl Ldap {
             class: TagClass::Application,
             inner: (),
         });
-        Ok(self.op_call(LdapOp::Unbind, req).await.map(|_| ())?)
+        self.op_call(LdapOp::Unbind, req).await.map(|_| ())
     }
 
     /// Return the message ID of the last active operation. When the handle is initialized, this
@@ -714,10 +847,7 @@ impl Ldap {
             class: TagClass::Application,
             inner: msgid as i64,
         });
-        Ok(self
-            .op_call(LdapOp::Abandon(msgid), req)
-            .await
-            .map(|_| ())?)
+        self.op_call(LdapOp::Abandon(msgid), req).await.map(|_| ())
     }
 
     /// Check whether the underlying connection has been closed.
