@@ -5,5 +5,5 @@ mod lang_mapping;
 mod script;
 
 pub use self::detect::detect_script;
-pub use self::detect::{raw_detect_script, RawScriptInfo};
+pub use self::detect::{RawScriptInfo, raw_detect_script};
 pub use self::script::Script;
