@@ -34,7 +34,7 @@ use crate::net::{
     protocol::network::Network,
     routing::{
         dispatcher::{face::FaceState, tables::Tables},
-        hat::{CurrentFutureTrait, HatTokenTrait, SendDeclare},
+        hat::{CurrentFutureTrait, HatTokenTrait, SendDeclare, Sources},
         router::{NodeId, Resource, SessionContext},
         RoutingContext,
     },
@@ -722,4 +722,48 @@ impl HatTokenTrait for HatCode {
             forget_simple_token(tables, face, id, send_declare)
         }
     }
+
+    fn get_tokens(&self, tables: &Tables) -> Vec<(Arc<Resource>, Sources)> {
+        // Compute the list of known tokens (keys)
+        hat!(tables)
+            .linkstatepeer_tokens
+            .iter()
+            .map(|s| {
+                // Compute the list of routers, peers and clients that are known
+                // sources of those tokens
+                let mut routers = vec![];
+                let mut peers = vec![];
+                let mut clients = vec![];
+                for t in &res_hat!(s).linkstatepeer_tokens {
+                    if let Some(whatami) = hat!(tables)
+                        .linkstatepeers_net
+                        .as_ref()
+                        .and_then(|net| net.get_node(t))
+                        .and_then(|n| n.whatami)
+                    {
+                        match whatami {
+                            WhatAmI::Router => routers.push(*t),
+                            WhatAmI::Peer => peers.push(*t),
+                            WhatAmI::Client => clients.push(*t),
+                        }
+                    } else {
+                        peers.push(*t);
+                    }
+                }
+                for ctx in s.session_ctxs.values().filter(|ctx| {
+                    !ctx.face.is_local && ctx.face.whatami == WhatAmI::Client && ctx.token
+                }) {
+                    clients.push(ctx.face.zid);
+                }
+                (
+                    s.clone(),
+                    Sources {
+                        routers,
+                        peers,
+                        clients,
+                    },
+                )
+            })
+            .collect()
+    }
 }
