@@ -569,24 +569,9 @@ impl HatBaseTrait for HatCode {
                     .unwrap()
                     .remove_link(&face.zid)
                 {
-                    pubsub_remove_node(
-                        &mut wtables,
-                        &removed_node.zid,
-                        WhatAmI::Router,
-                        send_declare,
-                    );
-                    queries_remove_node(
-                        &mut wtables,
-                        &removed_node.zid,
-                        WhatAmI::Router,
-                        send_declare,
-                    );
-                    token_remove_node(
-                        &mut wtables,
-                        &removed_node.zid,
-                        WhatAmI::Router,
-                        send_declare,
-                    );
+                    pubsub_remove_node(&mut wtables, &removed_node, WhatAmI::Router, send_declare);
+                    queries_remove_node(&mut wtables, &removed_node, WhatAmI::Router, send_declare);
+                    token_remove_node(&mut wtables, &removed_node, WhatAmI::Router, send_declare);
                 }
 
                 if hat!(wtables).full_net(WhatAmI::Peer) {
@@ -608,22 +593,17 @@ impl HatBaseTrait for HatCode {
                     {
                         pubsub_remove_node(
                             &mut wtables,
-                            &removed_node.zid,
+                            &removed_node,
                             WhatAmI::Peer,
                             send_declare,
                         );
                         queries_remove_node(
                             &mut wtables,
-                            &removed_node.zid,
-                            WhatAmI::Peer,
-                            send_declare,
-                        );
-                        token_remove_node(
-                            &mut wtables,
-                            &removed_node.zid,
+                            &removed_node,
                             WhatAmI::Peer,
                             send_declare,
                         );
+                        token_remove_node(&mut wtables, &removed_node, WhatAmI::Peer, send_declare);
                     }
 
                     hat_mut!(wtables).shared_nodes = shared_nodes(
@@ -672,19 +652,19 @@ impl HatBaseTrait for HatCode {
                             {
                                 pubsub_remove_node(
                                     tables,
-                                    &removed_node.zid,
+                                    &removed_node,
                                     WhatAmI::Router,
                                     send_declare,
                                 );
                                 queries_remove_node(
                                     tables,
-                                    &removed_node.zid,
+                                    &removed_node,
                                     WhatAmI::Router,
                                     send_declare,
                                 );
                                 token_remove_node(
                                     tables,
-                                    &removed_node.zid,
+                                    &removed_node,
                                     WhatAmI::Router,
                                     send_declare,
                                 );
@@ -707,19 +687,19 @@ impl HatBaseTrait for HatCode {
                                     for (_, removed_node) in changes.removed_nodes {
                                         pubsub_remove_node(
                                             tables,
-                                            &removed_node.zid,
+                                            &removed_node,
                                             WhatAmI::Peer,
                                             send_declare,
                                         );
                                         queries_remove_node(
                                             tables,
-                                            &removed_node.zid,
+                                            &removed_node,
                                             WhatAmI::Peer,
                                             send_declare,
                                         );
                                         token_remove_node(
                                             tables,
-                                            &removed_node.zid,
+                                            &removed_node,
                                             WhatAmI::Peer,
                                             send_declare,
                                         );
